import { computed, onUnmounted, ref } from '@vue/composition-api';
import { useStore } from './useStore';
export const useSelector = (select, equalityFn = (nextState, currentState) => nextState !== currentState) => {
    const { subscribe, getState } = useStore();
    const selector = ref(select(getState()));
    let currentState;
    const unsubscribe = subscribe(() => {
        const nextState = select(getState());
        if (equalityFn(nextState, currentState)) {
            currentState = nextState;
            selector.value = currentState;
        }
    });
    onUnmounted(unsubscribe);
    return computed(() => selector.value);
};
