import { Dispatch } from 'redux'
import { VNode } from 'vue'
import { useDispatch } from './hooks/useDispatch'
import { useSelector } from './hooks/useSelector'
import { Ref, computed } from '@vue/composition-api'

type MapStateToProps<State, StateProps> = (state: State) => StateProps
type MapDispatchToProps<DispatchProps> = (dispatch: Dispatch) => DispatchProps

type MergeProps<StateProps, DispatchProps, MergedProps = any> = (
  stateProps: StateProps,
  dispatchProps: DispatchProps,
) => MergedProps

interface Connect {
  <StateProps, DispatchProps>(
    mapStateToProps: MapStateToProps<any, StateProps>,
    mapDispatchToProps: MapDispatchToProps<DispatchProps>,
  ): (component: (props: StateProps & DispatchProps) => VNode) => VNode

  <StateProps, DispatchProps, MergedProps>(
    mapStateToProps: MapStateToProps<any, StateProps>,
    mapDispatchToProps: MapDispatchToProps<DispatchProps>,
    mergeProps: MergeProps<StateProps, DispatchProps, MergedProps>,
  ): (component: (props: MergedProps) => VNode) => VNode
}

export const connect: Connect = <StateProps, DispatchProps, MergedProps>(
  mapStateToProps: MapStateToProps<any, StateProps>,
  mapDispatchToProps: MapDispatchToProps<DispatchProps>,
  mergeProps?: MergeProps<StateProps, DispatchProps, MergedProps>,
) => {
  const dispatch = useDispatch()

  const dispatchProps = mapDispatchToProps(dispatch)

  const stateProps = useSelector(state => mapStateToProps(state))

  const mergedProps = computed(() =>
    (
      mergeProps ||
      ((stateProps, dispatchProps) => ({
        ...stateProps,
        ...dispatchProps,
      }))
    )(stateProps.value, dispatchProps),
  )

  return (
    component: (props: MergedProps | (StateProps & DispatchProps)) => VNode,
  ) => component(mergedProps.value)
}
