import CompositionApi, {
  createComponent,
  createElement as h,
} from '@vue/composition-api'
import VueReduxHooks, { connect } from '../../src'
import { createLocalVue } from '@vue/test-utils'
import { createReducer } from '@reduxjs/toolkit'
import { createStore } from 'redux'

describe('connect()', () => {
  it('takes mapStateToProps', () => {
    const localVue = createLocalVue()

    localVue.use(CompositionApi)
    const reducer = createReducer(0, {
      INCREMENT: state => state + 1,
    })

    const store = createStore(reducer)

    localVue.use(VueReduxHooks, store)

    const secondChild = createComponent({
      props: { INCREMENT: { default: null }, state: { default: null } },
      render: () => h('div'),
      //   setup: props => ({ state: props.state }),
    })

    const vm = new localVue({
      components: {
        child: createComponent({
          setup() {
            const connector = connect(
              (state: typeof reducer) => ({ state }),
              dispatch => ({
                INCREMENT: () =>
                  dispatch({
                    type: 'INCREMENT',
                  }),
              }),
            )

            return () => connector(props => h(secondChild, { props }))
          },
        }),
      },
      setup: () => () => h('child'),
    }).$mount()

    const child = (vm.$children[0].$children[0] as unknown) as {
      INCREMENT(): {
        type: string
      }
      state: number
    }

    expect(child.state).toStrictEqual(0)
    child.INCREMENT()
    expect(child.state).toStrictEqual(1)
  })
})
